"""Tests for pysmsboxnet."""

from __future__ import annotations

import random
from typing import Any

import aiohttp
import pytest
from pysmsboxnet import exceptions
from pysmsboxnet.api import Client

# Constants
SMSBOX_HOST = "api.smsbox.pro"
SMSBOX_API_KEY = "pub-xxxxx-xxxxx-xxxx-xxxx-xxxxx-xxxxxxxx"
SMS_RECIPIENT = "9999001"
SMS_MSG = "Test d'un message ! En plus il va être envoyé."
SEND_MODE = "expert"
SMSBOX_STRATEGY = "2"


@pytest.mark.asyncio
async def test_parameters_error(aresponses: Any) -> None:
    """Test if exception is raised if web server returns ERROR 01."""
    aresponses.add(
        "api.smsbox.pro",
        "/1.1/api.php",
        "post",
        aresponses.Response(
            text="ERROR 01",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(session, SMSBOX_HOST, SMSBOX_API_KEY)
        with pytest.raises(exceptions.ParameterErrorException):
            await sms.send(
                SMS_RECIPIENT,
                SMS_MSG,
                SEND_MODE,
                {"strategy": SMSBOX_STRATEGY},
            )
            await session.close()


@pytest.mark.asyncio
async def test_bad_auth(aresponses: Any) -> None:
    """Test if exception is raised in case of wrong authentication ERROR 02."""
    aresponses.add(
        "api.smsbox.pro",
        "/1.1/api.php",
        "post",
        aresponses.Response(
            text="ERROR 02",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        with pytest.raises(exceptions.AuthException):
            await sms.send(
                SMS_RECIPIENT,
                SMS_MSG,
                SEND_MODE,
                {"strategy": SMSBOX_STRATEGY},
            )
            await session.close()


@pytest.mark.asyncio
async def test_billing(aresponses: Any) -> None:
    """Test if BillingException is raised in case of ERROR 03."""
    aresponses.add(
        "api.smsbox.pro",
        "/1.1/api.php",
        "post",
        aresponses.Response(
            text="ERROR 03",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        with pytest.raises(exceptions.BillingException):
            await sms.send(
                SMS_RECIPIENT,
                SMS_MSG,
                SEND_MODE,
                {"strategy": SMSBOX_STRATEGY},
            )
            await session.close()


@pytest.mark.asyncio
async def test_bad_dest(aresponses: Any) -> None:
    """Test if WrongRecipientException is raised on ERROR 04."""
    aresponses.add(
        "api.smsbox.pro",
        "/1.1/api.php",
        "post",
        aresponses.Response(
            text="ERROR 04",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        with pytest.raises(exceptions.WrongRecipientException):
            await sms.send(
                SMS_RECIPIENT,
                SMS_MSG,
                SEND_MODE,
                {"strategy": SMSBOX_STRATEGY},
            )
            await session.close()


@pytest.mark.asyncio
async def test_internal_error(aresponses: Any) -> None:
    """Test if right exception is raised on ERROR 05."""
    aresponses.add(
        "api.smsbox.pro",
        "/1.1/api.php",
        "post",
        aresponses.Response(
            text="ERROR 05",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        with pytest.raises(exceptions.InternalErrorException):
            await sms.send(
                SMS_RECIPIENT,
                SMS_MSG,
                SEND_MODE,
                {"strategy": SMSBOX_STRATEGY},
            )
            await session.close()


@pytest.mark.asyncio
async def test_other_error(aresponses: Any) -> None:
    """Test unknown error."""
    aresponses.add(
        "api.smsbox.pro",
        "/1.1/api.php",
        "post",
        aresponses.Response(
            text="ERROR",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        with pytest.raises(exceptions.SMSBoxException):
            await sms.send(
                SMS_RECIPIENT,
                SMS_MSG,
                SEND_MODE,
                {"strategy": SMSBOX_STRATEGY},
            )
            await session.close()


@pytest.mark.asyncio
async def test_http_error(aresponses: Any) -> None:
    """Test if HTTP status is not 200."""
    aresponses.add(
        "api.smsbox.pro",
        "/1.1/api.php",
        "post",
        aresponses.Response(
            text="",
            status=500,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        with pytest.raises(exceptions.HTTPException):
            await sms.send(
                SMS_RECIPIENT,
                SMS_MSG,
                SEND_MODE,
                {"strategy": SMSBOX_STRATEGY},
            )
            await session.close()


@pytest.mark.asyncio
async def test_ok(aresponses: Any) -> None:
    """Test result OK without ID."""
    aresponses.add(
        "api.smsbox.pro",
        "/1.1/api.php",
        "post",
        aresponses.Response(
            text="OK",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        result = await sms.send(
            SMS_RECIPIENT,
            SMS_MSG,
            SEND_MODE,
            {"strategy": SMSBOX_STRATEGY},
        )
        assert 0 == result
        await session.close()


@pytest.mark.asyncio
async def test_ok_with_id(aresponses: Any) -> None:
    """Test result OK with a random ID."""
    # Get a random integer which will serv as the message ID
    msg_id = random.randint(100000000000, 999999999999)
    aresponses.add(
        "api.smsbox.pro",
        "/1.1/api.php",
        "post",
        aresponses.Response(
            text=f"OK {msg_id}",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        result = await sms.send(
            SMS_RECIPIENT,
            SMS_MSG,
            SEND_MODE,
            {"strategy": SMSBOX_STRATEGY, "id": "1"},
        )
        assert msg_id == result
        await session.close()


@pytest.mark.asyncio
async def test_credits(aresponses: Any) -> None:
    """Test credits async property returning a random number."""
    # Get a random float which will serv as the number of credits
    account_credits = round(random.uniform(0, 9999), 1)
    aresponses.add(
        "api.smsbox.pro",
        "/api.php",
        "post",
        aresponses.Response(
            text=f"CREDIT {account_credits}",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        result = await sms.get_credits()
        assert account_credits == result
        await session.close()


@pytest.mark.asyncio
async def test_exception_credits(aresponses: Any) -> None:
    """Test get credits async property raising exception."""
    aresponses.add(
        "api.smsbox.pro",
        "/api.php",
        "post",
        aresponses.Response(
            text="ERROR 02",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        with pytest.raises(exceptions.SMSBoxException):
            await sms.get_credits()
            await session.close()


@pytest.mark.asyncio
async def test_error_credits(aresponses: Any) -> None:
    """Test get credits async property returning strange string."""
    aresponses.add(
        "api.smsbox.pro",
        "/api.php",
        "post",
        aresponses.Response(
            text="FFF",
            status=200,
        ),
    )
    async with aiohttp.ClientSession() as session:
        sms = Client(
            session,
            SMSBOX_HOST,
            SMSBOX_API_KEY,
        )
        with pytest.raises(exceptions.SMSBoxException):
            await sms.get_credits()
            await session.close()



"""smsbox.net api client module."""

from __future__ import annotations

import logging
from http import client as http_client

from aiohttp import ClientSession

from . import exceptions

_LOGGER = logging.getLogger(__name__)


class Client:
    """API client class.

    :param aiohttp.ClientSession session: the aiohttp session to use
    :param str host: the API endpoint host, for example api.smsbox.pro (https is forced)
    :param str cle_api: the SMSBox API key, name is in French to reflect the documentation
    """

    def __init__(self, session: ClientSession, host: str, cle_api: str):
        """Initialize the SMS."""
        self.host = host
        self.cle_api = cle_api
        self.session = session

    async def __smsbox_request(self, uri: str, parameters: dict[str, str]) -> str:
        """Private method to send a request to the API.

        :param str uri: the host API endpoint, for example api.php or 1.1/api.php
        :param dict parameters: parameters to pass to the API

        :returns: SMSBox API response
        :rtype: str

        :raises pysmsboxnet.exceptions.HTTPException: if HTTP status is not 200 OK
        :raises pysmsboxnet.exceptions.SMSBoxException: SMSBox API returned ERROR
        :raises pysmsboxnet.exceptions.ParameterErrorException: bad parameters were passed to the API
        :raises pysmsboxnet.exceptions.AuthException: bad API key has been specified
        :raises pysmsboxnet.exceptions.BillingException: there are no enough credits to send the SMS
        :raises pysmsboxnet.exceptions.WrongRecipientException: recipient format is wrong
        :raises pysmsboxnet.exceptions.InternalErrorException: SMSBox API internal error
        """
        headers = {
            "authorization": f"App {self.cle_api}",
        }

        _LOGGER.debug(
            "Sending request to SMSBox API using host %s with URI %s and parameters %s",
            self.host,
            uri,
            parameters,
        )
        async with self.session.post(
            url=f"https://{self.host}/{uri}",
            headers=headers,
            data=parameters,
        ) as resp:
            _LOGGER.debug("HTTP response: %s", resp.status)
            if resp.status != http_client.OK:
                raise exceptions.HTTPException(resp.status)
            resp_text = await resp.text()
            _LOGGER.debug("API response: %s", resp_text)
            if resp_text == "ERROR":
                raise exceptions.SMSBoxException
            elif resp_text == "ERROR 01":
                raise exceptions.ParameterErrorException
            elif resp_text == "ERROR 02":
                raise exceptions.AuthException
            elif resp_text == "ERROR 03":
                raise exceptions.BillingException
            elif resp_text == "ERROR 04":
                raise exceptions.WrongRecipientException
            elif resp_text == "ERROR 05":
                raise exceptions.InternalErrorException
            else:
                return resp_text

    async def send(
        self, dest: str, msg: str, mode: str, parameters: dict[str, str] | None = None
    ) -> int:
        """Send a SMS.

        :param str dest: SMS recipient(s), see API documentation about how to format
        :param str msg: the SMS message
        :param str mode: send mode,  mode API parameter
        :param dict parameters: other API parameter as strategy or if other charset than UTF8 is needed

        :returns: SMS ID if id parameter is set to 1 else 0
        :rtype: int
        """
        post_data = {
            "dest": dest,
            "msg": msg,
            "mode": mode,
            "charset": "utf-8",
        }
        if parameters:
            post_data.update(parameters)

        resp_text = await self.__smsbox_request("1.1/api.php", post_data)

        resp_ok = resp_text.split(" ")
        if len(resp_ok) == 1:
            return 0
        return int(resp_ok[1])

    async def get_credits(self) -> float:
        """Return float number of credits.

        :raises pysmsboxnet.exceptions.SMSBoxException: result is not OK
        """
        post_data = {
            "action": "credit",
        }

        resp_text = await self.__smsbox_request("api.php", post_data)
        if resp_text.startswith("CREDIT"):
            return float(resp_text.split(" ")[1])
        else:
            raise exceptions.SMSBoxException(resp_text)



"""Copy example.py to a temporary directory."""

import os
import shutil

PROJECT_PATH = os.path.realpath(os.path.join(os.path.dirname(__file__), ".."))
EXAMPLE_PATH = os.path.join(PROJECT_PATH, "example.py")
TEMP_DIR = os.getenv("RUNNER_TEMP")
EXAMPLE_DEST = os.path.join(TEMP_DIR, "test_example")

os.mkdir(EXAMPLE_DEST)
shutil.copy(EXAMPLE_PATH, EXAMPLE_DEST)



"""Example script."""

import asyncio
from os import getenv
from sys import exit

import aiohttp
from pysmsboxnet import exceptions
from pysmsboxnet.api import Client


async def main():
    """Execute main instructions."""
    async with aiohttp.ClientSession() as session:
        sms = Client(session, "api.smsbox.pro", API_KEY)

        try:
            # To send a message
            # we define the strategy in the dict passed as last parameter
            # In this dict, we also ask the API to return the ID of message sent
            # With the parameter id set to 1
            # If we don't set id, the send function will return 0
            # In case of failure an exception will be thrown
            # Uncomment it if you need it
            # msgID = await sms.send(
            #     SMS_RECIPIENT, "Test message.", "expert", {"strategy": "2", "id": "1"}
            # )
            # We display the message is sent and its ID
            # print(f"SMS sent, ID : {msgID}")

            # We get remaining credits
            remaining_credits = await sms.get_credits()
            # print(f"Remaining credits: {remaining_credits}")
            if remaining_credits > 0:
                print("There are remaining credits")
            else:
                print("No remaining credit")
        except exceptions.SMSBoxException as e:
            print(f"Exception: {e}")
            await session.close()


# Get API key from environment variable
API_KEY = getenv("SMSBOX_API_KEY", "nokey")
SMS_RECIPIENT = "9990001"

if API_KEY == "nokey":
    print("No key specified, exiting")
    exit(1)

asyncio.run(main())



"""Python library to use smsbox.net API."""

try:
    from ._version import version as __version__  # type: ignore
    from ._version import version_tuple  # type: ignore
except ImportError:
    __version__ = "unknown version"
    version_tuple = (0, 0, "unknown version")
